/*
 * sensors.c
 *
 *  Created on: 11/ott/2013
 *      Author: nicholascolombo
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <wiiuse.h>
#include <time.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <netinet/in.h>
#include <netdb.h>
#include <fcntl.h>

#define TIME_MILLISEC	16


typedef const struct
{
	const signed short a;
	const signed short b;
	const signed short k;
}
ir_distance_sensor;

float ir_distance_calculate_cm(ir_distance_sensor sensor, unsigned short adc_value);
/* Configure MCU bits to use Sharp IR Sensors Board */
void setupSharp(void);
 /* Read value of Sharp sensor mapped on address "addr" */
uint16_t readSharp(unsigned char addr);
float readSharp_cm(unsigned char addr); /* Read value of Sharp sensor mapped on address "addr"  and convert to cm distance */

	float ir_distance_calculate_cm(ir_distance_sensor sensor, unsigned short adc_value){
		if (adc_value + sensor.b <= 0)
		{
			return -1;
		}
		float dist_cm;
		dist_cm = (float)sensor.a / (float)(adc_value + sensor.b) - (float)sensor.k;
		//dist_cm =  5461 / (adc_value + -17) - 2;
		//printf("cm%d ",dist_cm);
		return dist_cm;
	}

	void setupSharp(void){/* Configure MCU bits to use Sharp IR Sensors Board */
		//Configure MCU outputs to interface Board inputs
		DDRA |= (1<<PA2)|(1<<PA3);//MCU PIN PA2 and PA3 set as Output
		DDRF |= (1<<PF4) ;//MCU PIN PF4 set as Output
	}

	uint16_t readSharp(unsigned char addr){ /* Read value of Sharp sensor mapped on address "addr" */
	/* Note: All PORTA pin values are inverted (1 is LOW, 0 is HIGH)
	 * This is due to cm-510 output transistor, I guess
	 */
	switch(addr){
		case 0:	//MPLEX: ADDR 000 (0)
			PORTA = (1<<PA3)|(1<<PA2); //BIT1:0 = 00
			PORTF = (0<<PF5);//BIT2 = 0
			break;
		case 1:	//MPLEX: ADDR 001 (1)
			PORTA = (1<<PA3)|(0<<PA2); //BIT1:0 = 01
			PORTF = (0<<PF5);//BIT2 = 0
			break;
		case 2:	//MPLEX: ADDR 010 (2)
			PORTA = (0<<PA3)|(1<<PA2); //BIT1:0 = 10
			PORTF = (0<<PF5);//BIT2 = 0
			break;
		case 3:	//MPLEX: ADDR 011 (3)
			PORTA = (0<<PA3)|(0<<PA2); //BIT1:0 = 11
			PORTF = (0<<PF5);//BIT2 = 0
			break;
		case 4:	//MPLEX: ADDR 100 (4)
			PORTA = (1<<PA3)|(1<<PA2); //BIT1:0 = 00
			PORTF = (1<<PF5);//BIT2 = 1
			break;
		case 5:	//MPLEX: ADDR 101 (5)
			PORTA = (1<<PA3)|(0<<PA2); //BIT1:0 = 01
			PORTF = (1<<PF5);//BIT2 = 1
			break;
		case 6:	//MPLEX: ADDR 110 (6)
			PORTA = (0<<PA3)|(1<<PA2); //BIT1:0 = 10
			PORTF = (1<<PF5);//BIT2 = 1
			break;
		case 7:	//MPLEX: ADDR 111 (7)
			PORTA = (0<<PA3)|(0<<PA2); //BIT1:0 = 11
			PORTF = (1<<PF5);//BIT2 = 1
			break;
		default :
			printf("\nWrong Address (%d) Selected on channel 2\n", addr);
			return -1;
	}
	//Pause before acquiring..
	Timer1Write(0);
	int time = Timer1Read();
	while(time < TIME_MILLISEC ){
		time = Timer1Read();
		//printf("time=%d\n",time);
	}
	int16_t result = adc_start(6);	//Channel 6
	//filtering : pass only 0.3V (64) - 3V (613)
	if (result < 64)
		result = 64;
	if (result > 613)
		result = 613;
	//printf("result : \t%d\n",result);
	return result;
}

	unsigned short sharp_calib[17][2]={
	{63,40},
	{71,35},
	{85,30},
	{106,25},
	{132,20},
	{153,18},
	{165,16},
	{192,14},
	{214,12},
	{257,10},
	{286,9},
	{319,8},
	{360,7},
	{415,6},
	{480,5},
	{562,4},
	{613,3}
	};

	float readSharp_cm(unsigned char addr){ /* Read value of Sharp sensor mapped on address "addr"  and convert to cm distance */
		float val_cm;
		unsigned short val_10bit = readSharp(addr);
		int i;
		for(i = 0; i<17; i++){
			if (val_10bit >=  sharp_calib[i][0] && val_10bit < sharp_calib[i+1][0]){
				val_cm = (float)sharp_calib[i][1] + (float)((float)val_10bit - (float) sharp_calib[i][0])* (float)((float)((float)sharp_calib[i+1][1]- (float)sharp_calib[i][1])/(float)((float)sharp_calib[i+1][0] - (float)sharp_calib[i][0]));
				return val_cm;
			}
		}
		return -1; /* Error, data is out of range */
	}


